﻿using System;
using System.ComponentModel;
using System.Linq;
using Ascon.Pilot.SDK.CreateObjectSample;

namespace Ascon.Pilot.SDK.XpsDigitalSigner
{
    class ProgressDialogModel : INotifyPropertyChanged, IObserver<INotification>
    {
        private int _signedDocsCount;
        public int SignedDocsCount
        {
            get { return _signedDocsCount; }
            set
            {
                _signedDocsCount = value;
                OnPropertyChanged("SignedDocsCount");
            }
        }

        private readonly IPerson _currentPerson;
        private readonly IObjectsRepository _repository;
        private readonly BatchSigner _batchSigner;
        private int _totalDocsCount;
        private ProgressDialog _progressView;

        public int TotalDocsCount
        {
            get { return _totalDocsCount; }
            set
            {
                _totalDocsCount = value;
                OnPropertyChanged("TotalDocsCount");
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            handler?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        public ProgressDialogModel(BatchSigner batchSigner, IObservable<INotification> signatureNotifier, IObservable<INotification> fileNotifier, IObjectsRepository repository)
        {
            _repository = repository;
            _totalDocsCount = batchSigner.Count;
            _signedDocsCount = 0;
            _currentPerson = _repository.GetCurrentPerson();
            _batchSigner = batchSigner;
            signatureNotifier.Subscribe(this);
            fileNotifier.Subscribe(this);
        }

        public void UpdateProgress()
        {
            if (_progressView == null)
                return;
            SignedDocsCount = _signedDocsCount + 1;
            if (SignedDocsCount == TotalDocsCount)
            {
                if (_batchSigner.Count > 1)
                    _progressView.Close();
                _progressView = null;
                return;
            }
            _batchSigner.Next();
        }

        public void Run()
        {
            _progressView = new ProgressDialog { DataContext = this };
            _batchSigner.Start();
            if (_batchSigner.Count > 1)
                _progressView.ShowDialog();
        }

        public async void OnNext(INotification value)
        {
            if (value.ChangeKind == NotificationKind.ObjectSignatureChanged && value.ObjectId == _batchSigner.Current() && _currentPerson.Id == value.UserId)
            {
                var loaderOne = new ObjectLoader(_repository);
                var obj = await loaderOne.Load(value.ObjectId);
                
                if (obj.Files.Any(f => f.Name.Contains("Signature")))
                    UpdateProgress();
                
                return;
            }
            if (value.ChangeKind == NotificationKind.ObjectFileChanged && value.ObjectId == _batchSigner.Current() && _currentPerson.Id == value.UserId)
                UpdateProgress();
        }

        public void OnError(Exception error) { }
        public void OnCompleted() { }
    }
}
