SDK для серверных модулей расширения
===============


Требования
----------

Для создания серверных модулей расширения к Pilot-Server требуется один из следующих выпусков Visual Studio
- Visual Studio 2017 Professional Edition или старше

Создание и настройка проекта расширения
---------------------------------------

Чтобы создать новое расширение для Pilot-Server выполните следующие действия:

- Создайте проект типа Class Library (.NET Standard).
- Вызовите диалог настройки проекта. В поле AssemblyName добавьте к названию сборки `.servext` Например `PilotServerExtension.servext`
- Вызовите диалог редактирования информации о сборке. (Assembly Information...) Для этого нажмите на кнопку `Assembly Information...` и заполните необходимые поля.
    > Обязательно заполните поля Title, Product и Assembly version, т.к. Pilot-Server при загрузке вашего расширения будет искать информацию именно в этих полях.
- Подключите к проекту nuget пакеты System.ComponentModel.Composition и Ascon.Pilot.ServerExtensions.SDK.

Подключение SDK к проекту расширения
------------------------------------

Для подключения SDK Pilot к проекту расширения можно воспользоваться встроенным в Visual Studio механизмом распространения пакетов - NuGet Packages Manager.
Для того, чтобы подключить SDK с помощью Nuget Package Manager выполните следующие шаги:
- Вызовите [Package Manager Console](https://docs.nuget.org/docs/start-here/using-the-package-manager-console);
- Выполните команду:
```cpp
Install-Package Ascon.Pilot.ServerExtensions.SDK
```

- Щелкните правой кнопкой мыши на проекте и вызовите команду `Manage NuGet Packages`;
- Установите NuGet пакет Ascon.Pilot.ServerExtensions.SDK для вашего проекта.


## Содержание
[Система расширений](#typesOfExtensions)
- [Действие для серверных автоматизаций IServerActivity](#IServerActivity)
	- [IServerActivityContext](#IServerActivityContext)
	- [IAutomationEventContext](#IAutomationEventContext)

<a name="typesOfExtensions"/>
Система расширений
-------------------------

Система расширений основана на стандартном механизме расширений приложений в .NET - Managed Extensibility Framework (MEF). SDK содержит ряд экспортируемых и импортируемых интерфейсов. К экспортируемым интерфейсам относятся интерфейсы, которые описывают тип расширения. К импортируемым интерфейсам относятся интерфейсы взаимодействия с клиентом и работы с данными.

<a name="IServerActivity"/>
### Интерфейс IServerActivity ###

Данный интерфейс позволяет добавить новое действие, доступное в серверных скриптах автоматизаций.

```cpp
string IServerActivity.Name { get; };
```
Имя действия, используемое в скрипте автоматизации.

```cpp
Task IServerActivity.RunAsync(IModifierBase modifier, IModifierBackend backend, IServerActivityContext serverActivityContext, IAutomationEventContext automationEventContext);
```
Асинхронный метод обработки запуска действия.
где:
 - `modifier` - модификатор, позволяющий вносить изменения в объекты;
 - `backend` - обеспечивает доступ к данным системы;
 - `serverActivityContext` - контекст запуска действия, позволяющий получить параметры запуска действия;
 - `automationEventContext` - контекст события, позволяющий определить объект-источник события и другую дополнительную информацию. 

 
Пример IServerActivity:
```cpp
[Export(typeof(IServerActivity))]
public class SetLastWriteTimeActivity : IServerActivity
{
	public string Name => "SetLastWriteTime";

	public Task RunAsync(IModifierBase modifier, IModifierBackend backend, IServerActivityContext serverActivityContext, IAutomationEventContext automationEventContext)
	{
		modifier.EditObject(automationEventContext.Source).SetAttribute("LastWriteTime", DateTime.UtcNow);
		return Task.CompletedTask;
	}
}
```

<a name="IServerActivityContext"/>
### Интерфейс IServerActivityContext ###

Контекст запуска действия, позволяющий получить параметры запуска действия.

```cpp
Dictionary<string, object> IServerActivityContext.Params { get; }
```
Словарь параметров запуска действия, заданных в json скрипта автоматизации.

```cpp
List<string> IServerActivityContext.SourceTypes { get; }
```
Список типов, заданных в json скрипта автоматизации, для которых должно выполняться действие.

```cpp
TargetType? IServerActivityContext.Target { get; }
```
Определяет, должно ли действие быть применено к объекту-источнику события либо к связанным с ним объектам.

```cpp
RelationType? IServerActivityContext.RelationType { get; }
```
Определяет, к каким типам связанных объектов должно применяться действие (применимо только в случае IServerActivityContext.Target==TargetType.Relations).

```cpp
RelationFilterByChangeKind? IServerActivityContext.RelationFilterByChangeKind { get; } { get; }
```
Определяет, должно ли применяться действие ко всем связанным объектам, или же только к добавленным или удаленным.

```cpp
Dictionary<string, string> IServerActivityContext.Errors { get; }
```
Список ошибок при разборе json действия.

<a name="IAutomationEventContext"/>
### Интерфейс IAutomationEventContext ###

Контекст события-триггера действия.

```cpp
INObject IAutomationEventContext.Source { get; }
```
Объект-источник события.

```cpp
IEnumerable<INChange> IAutomationEventContext.Changes { get; }
```
Набор изменений, в результате которого сработал триггер. Включает в себя изменение по объекту-источнику события, если он был изменен.

```cpp
INPerson IAutomationEventContext.InitiatingPerson { get; }
```
Пользователь-инициатор события.

```cpp
DateTime IAutomationEventContext.EventDate { get; }
```
Время наступления события в формате UTC.