﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Xml.Serialization;

namespace Pilot.Xps.Domain.Annotations
{
    [Serializable]
    public class DAnnotationMessage
    {
        public DAnnotationMessage()
        {

        }

        public DAnnotationMessage(DAnnotationMessage annotationMessage)
        {
            Author = annotationMessage.Author;
            Text = annotationMessage.Text;
            DateTime = annotationMessage.DateTime;
            Id = annotationMessage.Id;
            Status = annotationMessage.Status;
            AnnotationId = annotationMessage.AnnotationId;
            AuthorUserId = annotationMessage.AuthorUserId;
        }

        protected bool Equals(DAnnotationMessage other)
        {
            bool equalStatuses;
            if (other.Status == null || Status == null)
                equalStatuses = ReferenceEquals(Status, other.Status);
            else
                equalStatuses = Equals(Status.StatusName, other.Status.StatusName) && Equals(Status.IconString, other.Status.IconString);
            return equalStatuses 
                   && string.Equals(Author, other.Author) 
                   && string.Equals(Text, other.Text) 
                   && DateTime.Equals(other.DateTime) 
                   && Id.Equals(other.Id) 
                   && AnnotationId.Equals(other.AnnotationId) 
                   && AuthorUserId.Equals(other.AuthorUserId);
        }

        public override int GetHashCode()
        {
            unchecked
            {
                var hashCode = (Author != null ? Author.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ (Text != null ? Text.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ DateTime.GetHashCode();
                hashCode = (hashCode * 397) ^ Id.GetHashCode();
                hashCode = (hashCode * 397) ^ AnnotationId.GetHashCode();
                hashCode = (hashCode * 397) ^ AuthorUserId.GetHashCode();
                hashCode = (hashCode * 397) ^ (Status != null ? Status.StatusName.GetHashCode() : 0);
                hashCode = (hashCode * 397) ^ (Status != null && Status.IconString != null ? Status.IconString.GetHashCode() : 0);
                return hashCode;
            }
        }

        public string Author { get; set; }

        public string Text { get; set; }

        public DateTime DateTime { get; set; }

        public Guid Id { get; set; }

        public Guid AnnotationId { get; set; }

        public Guid AuthorUserId { get; set; }

        public AnnotationStatus Status { get; set; }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != this.GetType()) return false;
            return Equals((DAnnotationMessage)obj);
        }
    }

    public class AnnotationMessages
    {
        public List<DAnnotationMessage> Messages = new List<DAnnotationMessage>();

        public void Serialize(Stream outStream)
        {
            using (var writer = new StreamWriter(outStream))
            {
                var xmlSerializer = new XmlSerializer(typeof(AnnotationMessages));
                xmlSerializer.Serialize(writer, this);
                writer.Flush();
            }
        }

        public static AnnotationMessages Deserialize(Stream stream)
        {
            var serializer = new XmlSerializer(typeof(AnnotationMessages));
            return serializer.Deserialize(stream) as AnnotationMessages;
        }
    }

    public class AnnotationStatus
    {
        public AnnotationStatus()
        {
        }

        public AnnotationStatus(string statusName, string iconString)
        {
            StatusName = statusName;
            IconString = iconString;
        }

        public string StatusName { get; set; }
        public string IconString { get; set; }
    }
}
