﻿using System;
using Ascon.Pilot.Server.Api.Contracts;
using Ascon.Pilot.Transport;

namespace Pilot.Xps.Connection.Client
{
    public class ServerCommandCallService<T> : ICallService
    {
        private readonly IRepository _repository;
        private readonly string _commandName;
        
        public ServerCommandCallService(IRepository repository, string processorName = null)
        {
            _repository = repository;
            _commandName = CommandNameParser.GetCommandName(typeof(T).Name, processorName);
        }

        public byte[] Call(byte[] data)
        {
            var result = _repository.InvokeServerCommand(_commandName, data)
                .GetAwaiter()
                .GetResult();

            switch (result.Result)
            {
                case ServerCommandResult.Success:
                    return result.Data;
                case ServerCommandResult.Error:
                    throw TransportClient.ReadException(result.Data);
                default:
                    throw new InvalidOperationException("Unable to deserialize server command response");
            }
        }
    }
}
